﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using Infragistics.Controls.Grids;
using Infragistics.DragDrop;
using Infragistics.Controls.Grids.Primitives;
using Infragistics.Windows;

namespace xamGrid_DragAndDrop_CS
{
    /// <summary>
    /// MainWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();

            this.xamGrid1.CellControlAttached += XamGrid1_CellControlAttached;

            #region Initialize data
            var items = new ObservableCollection<Item>();
            for(int i = 0; i < 10; i++)
            {
                items.Add(new Item { ID = i, Name = "Name" + i.ToString(), Note = "Note" + i.ToString() });
            }
            #endregion

            this.DataContext = items;
        }

        private void XamGrid1_CellControlAttached(object sender, CellControlAttachedEventArgs e)
        {
            #region Attach Drag and Drop framework to XamDataGrid rows
            DragSource dragSource = new DragSource();
            dragSource.IsDraggable = true;
            dragSource.DragStart += DragSource_DragStart;
            dragSource.Drop += DragSource_Drop;
            DragDropManager.SetDragSource(e.Cell.Row.Control, dragSource);

            DropTarget dropTarget = new DropTarget();
            dropTarget.IsDropTarget = true;
            DragDropManager.SetDropTarget(listBox1, dropTarget);
            #endregion
        }

        private void DragSource_DragStart(object sender, DragDropStartEventArgs e)
        {
            //Allow rows dragging when the mouse captures the row selector icon
            var elem = e.OriginalDragSource as Path;
            if (elem != null && elem.Name == "ActiveIndicator")
            {
                foreach (var item in xamGrid1.Rows)
                {
                    CheckBox cb = Utilities.GetDescendantFromName(item.Control, "checkBox1") as CheckBox;
                    if (cb.IsChecked == true)
                    {
                        this.xamGrid1.SelectionSettings.SelectedRows.Add(item);
                    }
                }
            }
            else
            {
                e.Cancel = true;
            }
        }

        private void DragSource_Drop(object sender, DropEventArgs e)
        {
            #region move rows when rows are dropped to ListBox
            var sourceRows = this.xamGrid1.SelectionSettings.SelectedRows.OrderBy(x=>((Item)x.Data).ID).ToList();

            for(int i = 0; i < sourceRows.Count; i++)
            {
                //Move selected items to ListBox
                listBox1.Items.Add(sourceRows[i].Data as Item);

                //Remove moved items from XamGrid
                var dc = this.xamGrid1.ItemsSource as ObservableCollection<Item>;
                dc.Remove(sourceRows[i].Data as Item);
            }
            #endregion
        }

        private void CheckBox_Checked(object sender, RoutedEventArgs e)
        {
            int index = ((sender as CheckBox).Tag as RowSelectorCellControl).Cell.Row.Index;
            xamGrid1.SelectionSettings.SelectedRows.Add(xamGrid1.Rows[index]);

            //var rowSelectorCell = Utilities.GetAncestorFromType(
            //    sender as DependencyObject,
            //    typeof(RowSelectorCellControl),
            //    true
            //    );
            //var row = ((CellControlBase)rowSelectorCell).Cell.Row as Row;
            
            //this.xamGrid1.SelectionSettings.SelectedRows.Add(row);
        }

        private void CheckBox_Unchecked(object sender, RoutedEventArgs e)
        {
            if (xamGrid1.SelectionSettings.SelectedRows.Count > 0)
            {
                int index = ((sender as CheckBox).Tag as RowSelectorCellControl).Cell.Row.Index;
                xamGrid1.SelectionSettings.SelectedRows.Remove(xamGrid1.Rows[index]);
            }
        }

        private void CheckBox_PreviewMouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            (sender as CheckBox).IsChecked = !(sender as CheckBox).IsChecked;
        }

        private void xamGrid1_CellClicked(object sender, CellClickedEventArgs e)
        {
            CheckBox cb1 = Utilities.GetDescendantFromName(e.Cell.Row.Control, "checkBox1") as CheckBox;
            cb1.IsChecked = true;
        }
    }

    public class Item
    {
        public int ID { get; set; }
        public string Name { get; set; }
        public string Note { get; set; }
    }
}
